package com.mortals.xhx.module.pj.web;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.common.pdu.RespData;
import com.mortals.xhx.common.pdu.site.SitePdu;
import com.mortals.xhx.feign.site.ISiteFeign;
import com.mortals.xhx.module.pj.model.PjEvaluateQuery;
import com.mortals.xhx.module.pj.model.PjEvaluateStatQuery;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StopWatch;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.*;
import com.mortals.framework.model.Context;

import com.mortals.xhx.module.pj.model.PjEvaluateStatEntity;
import com.mortals.xhx.module.pj.service.PjEvaluateStatService;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.mortals.xhx.common.code.*;

/**
 * 评价汇总统计
 *
 * @author zxfei
 * @date 2024-06-24
 */
@RestController
@RequestMapping("pj/evaluate/stat")
@Slf4j
public class PjEvaluateStatController extends BaseCRUDJsonBodyMappingController<PjEvaluateStatService, PjEvaluateStatEntity, Long> {

    @Autowired
    private ISiteFeign siteFeign;

    public PjEvaluateStatController() {
        super.setModuleDesc("评价汇总统计");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        this.addDict(model, "pjOption", PjOptionEnum.getEnumMap());
        this.addDict(model, "pjOptionList", PjOptionEnum.getEnumMap());

        //通过站点 查询存在的部门 大厅等
        Object siteId = model.get("siteId");
        if (!ObjectUtils.isEmpty(siteId)) {
            PjEvaluateStatQuery pjEvaluateStatQuery = new PjEvaluateStatQuery();
            pjEvaluateStatQuery.setSiteId((Long) siteId);
            List<PjEvaluateStatEntity> pjEvaluateStatEntities = this.service.find(pjEvaluateStatQuery, context);
            Map<String, String> sectionNameMap = pjEvaluateStatEntities.stream().filter(f -> !ObjectUtils.isEmpty(f.getSectionName())).collect(Collectors.toMap(PjEvaluateStatEntity::getSectionName, PjEvaluateStatEntity::getSectionName, (k1, k2) -> k2));
            Map<String, String> hallNameMap = pjEvaluateStatEntities.stream().filter(f -> !ObjectUtils.isEmpty(f.getHallName())).collect(Collectors.toMap(PjEvaluateStatEntity::getHallName, PjEvaluateStatEntity::getHallName, (k1, k2) -> k2));
            Map<String, String> windowFromnumMap = pjEvaluateStatEntities.stream().filter(f -> !ObjectUtils.isEmpty(f.getWindowFromnum())).collect(Collectors.toMap(PjEvaluateStatEntity::getWindowFromnum, PjEvaluateStatEntity::getWindowFromnum, (k1, k2) -> k2));

            this.addDict(model, "sectionNameList", sectionNameMap);
            this.addDict(model, "hallNameList", hallNameMap);
            this.addDict(model, "windowFromnumList", windowFromnumMap);
            this.addDict(model, "groupList", GroupListEnum.getEnumMap());
        }

        super.init(model, context);
    }


    @Override
    protected void doListBefore(PjEvaluateStatEntity query, Map<String, Object> model, Context context) throws AppException {
        if (ObjectUtils.isEmpty(query.getHallName())) query.setHallName("");
        if (ObjectUtils.isEmpty(query.getSectionName())) query.setSectionName("");
        if (ObjectUtils.isEmpty(query.getPjOption())) query.setPjOption("");
        if (ObjectUtils.isEmpty(query.getWindowFromnum())) query.setWindowFromnum("");

        //年月日

        //根据查询条件自动添加分组字段
        addGroup(query);

        model.put("siteId", query.getSiteId());

    }

    private static void addGroup(PjEvaluateStatEntity query) {
        if (!ObjectUtils.isEmpty(query.getSiteId())) {
            List<String> groupList = query.getGroupList();
            if (!groupList.contains("siteId")) {
                groupList.add("siteId");
            }
        }

        if (!ObjectUtils.isEmpty(query.getSectionName())
                ||!ObjectUtils.isEmpty(query.getSectionNameList())
                ||!ObjectUtils.isEmpty(query.getSectionNameNotList())) {
            List<String> groupList = query.getGroupList();
            if (!groupList.contains("sectionName")) {
                groupList.add("sectionName");
            }
        }
        if (!ObjectUtils.isEmpty(query.getHallName())
                ||!ObjectUtils.isEmpty(query.getHallNameList())
                ||!ObjectUtils.isEmpty(query.getHallNameNotList())) {
            List<String> groupList = query.getGroupList();
            if (!groupList.contains("hallName")) {
                groupList.add("hallName");
            }
        }

        if (!ObjectUtils.isEmpty(query.getWindowFromnum())
                ||!ObjectUtils.isEmpty(query.getWindowFromnumList())
                ||!ObjectUtils.isEmpty(query.getWindowFromnumNotList())) {
            List<String> groupList = query.getGroupList();
            if (!groupList.contains("windowFromnum")) {
                groupList.add("windowFromnum");
            }
        }

        if (!ObjectUtils.isEmpty(query.getPjOption())
                ||!ObjectUtils.isEmpty(query.getPjOptionList())
                ||!ObjectUtils.isEmpty(query.getPjOptionNotList())) {
            List<String> groupList = query.getGroupList();
            if (!groupList.contains("pjOption")) {
                groupList.add("pjOption");
            }
        }

    }

    @PostMapping(value = "/stat")
    @UnAuth
    public Rest<String> pjStat(@RequestBody PjEvaluateQuery pjEvaluateQuery) {
        Rest<String> ret = new Rest();
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        String busiDesc = "查询" + this.getModuleDesc();
        int code = 1;
        try {
            //天数区间分段计算
            DateTime attendStart = DateUtil.parseDate(pjEvaluateQuery.getPjTimeStart());
            DateTime attendEnd = DateUtil.parseDate(pjEvaluateQuery.getPjTimeEnd());
            Long compare = DateUtil.between(attendEnd, attendStart, DateUnit.DAY);
            StopWatch stopWatch = new StopWatch("stopwatch");
            log.info("计算天数区间：{}", compare);
            for (int i = 0; i <= compare.intValue(); i++) {
                DateTime curDate = DateUtil.offsetDay(attendStart, i);
                log.info("记录日期：{}", curDate.toDateStr());
                stopWatch.start("执行本地方法");
                SitePdu sitePdu = new SitePdu();
                sitePdu.setSize(-1);
                Rest<RespData<List<SitePdu>>> resp = siteFeign.list(sitePdu);
                if (resp.getCode() == 1) {
                    List<SitePdu> sitePduList = resp.getData().getData();
                    sitePduList.stream().forEach(site -> {
                        PjEvaluateStatEntity sitestatEntity = new PjEvaluateStatEntity();
                        sitestatEntity.initAttrValue();
                        sitestatEntity.setSiteId(site.getId());
                        sitestatEntity.setSiteName(site.getSiteName());
                        sitestatEntity.setSiteCode(site.getSiteCode());

                        sitestatEntity.setYear(curDate.year());
                        sitestatEntity.setMonth(curDate.month() + 1);
                        sitestatEntity.setDay(curDate.dayOfMonth());
                        //设置年月日
                        this.service.updateSitePjStat(sitestatEntity, context);
                    });
                }

                stopWatch.stop();
                log.info("日期：{} 完成,耗时：{}ms", curDate.toDateStr(), stopWatch.getLastTaskTimeMillis());
            }
            model.put("message_info", busiDesc + "成功");
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception e) {
            code = -1;
            this.doException(this.request, busiDesc, model, e);
            model.put("message_info", e.getMessage());
        }

        this.init(model, context);
        ret.setCode(code);
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;

    }


}