package com.mortals.xhx.module.stat.web;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.service.ICacheService;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.common.pdu.RespData;
import com.mortals.xhx.common.pdu.site.SitePdu;
import com.mortals.xhx.feign.site.ISiteFeign;
import com.mortals.xhx.module.pj.model.PjEvaluateQuery;
import com.mortals.xhx.module.pj.model.PjEvaluateStatEntity;
import com.mortals.xhx.module.stat.model.StatQuery;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StopWatch;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.*;
import com.mortals.framework.model.Context;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.module.stat.model.StatEntity;
import com.mortals.xhx.module.stat.service.StatService;
import org.apache.commons.lang3.ArrayUtils;
import com.mortals.framework.util.StringUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.alibaba.fastjson.JSONObject;

import java.util.Arrays;

import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import static com.mortals.framework.ap.SysConstains.*;

import com.mortals.xhx.common.code.*;

/**
 * 汇总统计
 *
 * @author zxfei
 * @date 2024-07-02
 */
@RestController
@RequestMapping("stat")
@Slf4j
public class StatController extends BaseCRUDJsonBodyMappingController<StatService, StatEntity, Long> {


    @Autowired
    private ISiteFeign siteFeign;


    public StatController() {
        super.setModuleDesc("汇总统计");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        super.init(model, context);
    }


    @PostMapping(value = "/stat")
    @UnAuth
    public Rest<String> stat(@RequestBody StatQuery query) {
        Rest<String> ret = new Rest();
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        String busiDesc = "查询" + this.getModuleDesc();
        int code = 1;
        try {
            //天数区间分段计算
            DateTime attendStart = DateUtil.parseDate(query.getCreateTimeStart());
            DateTime attendEnd = DateUtil.parseDate(query.getCreateTimeEnd());
            Long compare = DateUtil.between(attendEnd, attendStart, DateUnit.DAY);
            StopWatch stopWatch = new StopWatch("stopwatch");
            log.info("计算天数区间：{}", compare);
            for (int i = 0; i <= compare.intValue(); i++) {
                DateTime curDate = DateUtil.offsetDay(attendStart, i);
                log.info("记录日期：{}", curDate.toDateStr());
                stopWatch.start("执行本地方法");
                SitePdu sitePdu = new SitePdu();
                sitePdu.setSize(-1);
                Rest<RespData<List<SitePdu>>> resp = siteFeign.list(sitePdu);
                if (resp.getCode() == 1) {
                    List<SitePdu> sitePduList = resp.getData().getData();
                    sitePduList.stream().forEach(site -> {
                        StatEntity statEntity = new StatEntity();
                        statEntity.initAttrValue();
                        statEntity.setSiteId(site.getId());
                        statEntity.setSiteName(site.getSiteName());
                        statEntity.setSiteCode(site.getSiteCode());

                        statEntity.setYear(curDate.year());
                        statEntity.setMonth(curDate.month() + 1);
                        statEntity.setDay(curDate.dayOfMonth());
                        //设置年月日
                        this.service.updateSiteStat(statEntity, context);
                    });
                }

                stopWatch.stop();
                log.info("日期：{} 完成,耗时：{}ms", curDate.toDateStr(), stopWatch.getLastTaskTimeMillis());
            }
            model.put("message_info", busiDesc + "成功");
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception e) {
            code = -1;
            this.doException(this.request, busiDesc, model, e);
            model.put("message_info", e.getMessage());
        }

        this.init(model, context);
        ret.setCode(code);
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;

    }

}